package com.learnweblogic.examples.ch7.queue;

import com.learnweblogic.examples.BaseClient;

import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.NamingException;

public final class MessageProducer extends BaseClient {

  private QueueSender sender;
  private TextMessage msg;

  public MessageProducer(String [] argv)
    throws NamingException, JMSException
  {
    super(argv);

    Context ctx = getInitialContext();

    QueueConnectionFactory qConFactory = (QueueConnectionFactory)
      ctx.lookup("weblogic.jms.ConnectionFactory"); 

    Queue messageQueue = (Queue) ctx.lookup("MessageQueue");

    QueueConnection qCon = qConFactory.createQueueConnection();

    QueueSession session = qCon.createQueueSession(
      false, /* not a transacted session */
      Session.AUTO_ACKNOWLEDGE 
    );

    sender = session.createSender(messageQueue);

    msg = session.createTextMessage();

  }
 
  public void runClient() 
    throws JMSException
  {

    msg.setText("Hello");
    sender.send(msg);

    msg.setText("Welcome to JMS");
    sender.send(msg);

  }

  public static void main(String [] argv) 
    throws Exception
  {
    try {
      MessageProducer mp = new MessageProducer(argv);
      mp.runClient();
    } catch (NamingException ne) {

      System.err.println("");

      System.err.println("** Please ensure that you have setup the"+
        " JMS Server properly.  The JMS Server and the appropriate "+
        "JMS Destinations must be configured before running the "+
        "examples.");
      System.err.println("");

      throw ne;
    }

  }


}

