package com.learnweblogic.examples.ch10.textmessage;

import com.learnweblogic.examples.BaseClient;

import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.QueueConnectionFactory;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.NamingException;

public final class TextMessageSenderClient extends BaseClient {
    private static final String JMS_CONN_FACTORY =
        "weblogic.jms.ConnectionFactory";
    private String JMS_QUEUE = "MessageQueue";

    private QueueConnection queueConnection = null;
    private QueueSender queueSender = null;
    private QueueSession queueSession = null;
    private Queue queue = null;
    private TextMessage msg = null;

    public TextMessageSenderClient(String[] argv) throws Exception {
        super(argv);
        try {
            Context ctx = getInitialContext();

            QueueConnectionFactory factory =
                (QueueConnectionFactory) ctx.lookup(JMS_CONN_FACTORY);

            queueConnection = factory.createQueueConnection();

            // Create a non-transacted JMS Session
            queueSession =
                queueConnection.createQueueSession(
                    false,
                    Session.AUTO_ACKNOWLEDGE);

            queue = (Queue) ctx.lookup(JMS_QUEUE);

            queueSender = queueSession.createSender(queue);

            msg = queueSession.createTextMessage();

            queueConnection.start();

        } catch (Exception e) {
            System.err.println(
                "Error while attempting to "
                    + "connect to the server and look up the JMS"
                    + " QueueConnectionFactory.");
            System.err.println(
                "Please make sure that you have"
                    + " deployed the JMS Queue and specified the correct"
                    + " server URL.");

            e.printStackTrace();

            throw e;
        }
    }

    public void send(String message) throws JMSException {
        try {
            msg.setText(message);
            queueSender.send(msg);
        } catch (JMSException e) {
            System.err.println(
                "Exception raised while sending" + "to queue: " + JMS_QUEUE);
            e.printStackTrace();
            throw e;
        }
    }
    
	public static void main(String[] argv) throws Exception {

		TextMessageSenderClient tmsc = new TextMessageSenderClient(argv);
		tmsc.send("Hello");
	}

}